@@ -1034,13 +1034,12 @@ If the arglist is not available, return :NOT-AVAILABLE."))
 
 (defmethod arglist-dispatch ((operator (eql 'define-compiler-macro)) arguments)
   (match (cons operator arguments)
-    (('define-compiler-macro (#'function-exists-p gf-name) . _)
-     (let ((gf (fdefinition gf-name)))
-       (with-available-arglist (arglist) (decode-arglist (arglist gf))
-         (return-from arglist-dispatch
-           (make-arglist :provided-args (list gf-name)
-                         :required-args (list arglist)
-                         :rest "body" :body-p t)))))
+    (('define-compiler-macro (#'function-exists-p fun-name) . _)
+     (with-available-arglist (arglist) (decode-arglist (arglist fun-name))
+       (return-from arglist-dispatch
+         (make-arglist :provided-args (list fun-name)
+                       :required-args (list arglist)
+                       :rest "body" :body-p t))))
     (_)) ; Fall through
   (call-next-method))
 
