@@ -137,7 +137,8 @@ If you copy the HyperSpec to another location, customize the variable
     (forward-line)))
 
 (defun common-lisp-hyperspec--parse-map-file (file)
-  (with-current-buffer (find-file-noselect file)
+  (with-temp-buffer
+    (insert-file-contents file)
     (goto-char (point-min))
     (let ((result '()))
       (while (< (point) (point-max))
@@ -1316,7 +1317,7 @@ If you copy the HyperSpec to another location, customize the variable
   "Function that creates a URL for a glossary term.")
 
 (define-obsolete-variable-alias 'common-lisp-glossary-fun
-  'common-lisp-hyperspec-glossary-function)
+  'common-lisp-hyperspec-glossary-function "2015-12-29")
 
 (defvar common-lisp-hyperspec--glossary-terms (make-hash-table :test #'equal)
   "Collection of glossary terms and relative URLs.")
