@@ -26,8 +26,7 @@
 
 ;;;; Tests
 (require 'slime)
-(require 'ert nil t)
-(require 'ert "lib/ert" t) ;; look for bundled version for Emacs 23
+(require 'ert)
 (require 'cl-lib)
 (require 'bytecomp) ; byte-compile-current-file
 
@@ -48,7 +47,7 @@
 Exits Emacs when finished. The exit code is the number of failed tests."
   (interactive)
   (let ((ert-debug-on-error nil)
-        (timeout 30)
+        (timeout 60)
         (slime-background-message-function #'ignore))
     (slime)
     ;; Block until we are up and running.
@@ -60,7 +59,7 @@ Exits Emacs when finished. The exit code is the number of failed tests."
         (when timed-out
           (when noninteractive
             (kill-emacs 252)))))
-    (slime-sync-to-top-level 5)
+    (slime-sync-to-top-level 30)
     (let* ((selector (if randomize
                          `(member ,@(slime-shuffle-list
                                      (ert-select-tests (or test-name t) t)))
@@ -200,9 +199,8 @@ conditions (assertions)."
 (defun slime-wait-condition (name predicate timeout)
   (let ((end (time-add (current-time) (seconds-to-time timeout))))
     (while (not (funcall predicate))
-      (let ((now (current-time)))
-        (message "waiting for condition: %s [%s.%06d]" name
-                 (format-time-string "%H:%M:%S" now) (cl-third now)))
+      (message "waiting for condition: %s [%s]" name
+               (format-time-string "%H:%M:%S.%6N"))
       (cond ((time-less-p end (current-time))
              (error "Timeout waiting for condition: %S" name))
             (t
@@ -735,9 +733,10 @@ Confirm that SUBFORM is correctly located."
 (def-slime-test utf-8-source
     (input output)
     "Source code containing utf-8 should work"
-    (list (let*  ((bytes "\343\201\212\343\201\257\343\202\210\343\201\206")
-                  ;;(encode-coding-string (string #x304a #x306f #x3088 #x3046)
-                  ;;                      'utf-8)
+    (list (let*  ((bytes "\000\343\201\212\343\201\257\343\202\210\343\201\206")
+                  ;; (encode-coding-string
+                  ;;   (string #x0000 #x304a #x306f #x3088 #x3046)
+                  ;;   'utf-8)
                   (string (decode-coding-string bytes 'utf-8-unix)))
             (cl-assert (equal bytes (encode-coding-string string 'utf-8-unix)))
             (list (concat "(defun cl-user::foo () \"" string "\")")
@@ -1177,8 +1176,11 @@ on *DEBUGGER-HOOK*."
 (def-slime-test end-of-file
     (expr)
     "Signalling END-OF-FILE should invoke the debugger."
-    '(((cl:error 'cl:end-of-file))
+    '(((cl:error 'cl:end-of-file :stream cl:*standard-input*))
       ((cl:read-from-string "")))
+  (when (and noninteractive
+             (equal (slime-lisp-implementation-name) "ccl"))
+    (slime-skip-test "potential deadlocks"))
   (let ((value (slime-eval
                 `(cl:let ((condition nil))
                          (cl:with-simple-restart
@@ -1367,7 +1369,7 @@ Reconnect afterwards."
                 (die "Unexpected error running takeoff forms"
                      err)))
              (with-timeout
-                 (20
+                 (60
                   (die "Timeout waiting for recipe test to finish."
                        takeoff))
                (while t (sit-for 1)))))))
