@@ -283,7 +283,7 @@ form suitable for testing with #+."
                (t start)))
         ((<= code #x7ff) (utf8-encode-aux code buffer start end 2))
         ((<= #xd800 code #xdfff)
-         (%utf8-encode (code-char #xFFFD) ;; Replacement_Character
+         (%utf8-encode #xFFFD ;; Replacement_Character
                        buffer start end))
         ((<= code #xffff) (utf8-encode-aux code buffer start end 3))
         ((<= code #x1fffff) (utf8-encode-aux code buffer start end 4))
@@ -637,12 +637,12 @@ The stream calls READ-STRING when input is needed.")
 
 (defvar *auto-flush-interval* 0.2)
 
-(defun auto-flush-loop (stream interval &optional receive)
+(defun auto-flush-loop (stream interval &optional receive (flush #'force-output))
   (loop
    (when (not (and (open-stream-p stream)
                    (output-stream-p stream)))
      (return nil))
-   (force-output stream)
+   (funcall flush stream)
    (when receive
      (receive-if #'identity))
    (sleep interval)))
@@ -652,6 +652,11 @@ The stream calls READ-STRING when input is needed.")
   (spawn (lambda () (auto-flush-loop stream *auto-flush-interval* nil))
          :name "auto-flush-thread"))
 
+(definterface really-finish-output (stream)
+  "Make an auto-flush thread"
+  (spawn (lambda () (auto-flush-loop stream *auto-flush-interval* nil))
+         :name "auto-flush-thread"))
+
 
 ;;;; Documentation
 
@@ -805,9 +810,9 @@ forms."
             (compile nil `(lambda () ,expansion))))
       (values macro-forms compiler-macro-forms))))
 
-(definterface format-string-expand (control-string)
+(definterface format-string-expand (control-string &optional env)
   "Expand the format string CONTROL-STRING."
-  (macroexpand `(formatter ,control-string)))
+  (macroexpand `(formatter ,control-string) env))
 
 (definterface describe-symbol-for-emacs (symbol)
    "Return a property list describing SYMBOL.
@@ -1584,3 +1589,7 @@ Implementations intercept calls to SPEC and call, in this order:
                 nil)
                (prop-value t)
                (t nil)))))
+
+(definterface augment-features ()
+  "*features* or something else "
+  *features*)
