@@ -53,10 +53,12 @@
                     :accessor flush-scheduled)))
 
 (defun maybe-schedule-flush (stream)
-  (when (and (flush-thread stream)
-             (not (flush-scheduled stream)))
-    (setf (flush-scheduled stream) t)
-    (send (flush-thread stream) t)))
+  (when (flush-thread stream)
+    (or (flush-scheduled stream)
+        (progn
+          (setf (flush-scheduled stream) t)
+          (send (flush-thread stream) t)
+          t))))
 
 (defmacro with-slime-output-stream (stream &body body)
   `(with-slots (lock output-fn buffer fill-pointer column) ,stream
@@ -70,7 +72,7 @@
     (when (char= #\newline char)
       (setf column 0))
     (if (= fill-pointer (length buffer))
-        (finish-output stream)
+        (%stream-finish-output stream)
         (maybe-schedule-flush stream)))
   char)
 
@@ -83,7 +85,7 @@
            (count (- end start))
            (free (- len fill-pointer)))
       (when (>= count free)
-        (stream-finish-output stream))
+        (%stream-finish-output stream))
       (cond ((< count len)
              (replace buffer string :start1 fill-pointer
                       :start2 start :end2 end)
@@ -101,7 +103,10 @@
 (defmethod stream-line-column ((stream slime-output-stream))
   (with-slime-output-stream stream column))
 
-(defmethod stream-finish-output ((stream slime-output-stream))
+(defun reset-stream-line-column (stream)
+  (with-slime-output-stream stream (setf column 0)))
+
+(defun %stream-finish-output (stream)
   (with-slime-output-stream stream
     (unless (zerop fill-pointer)
       (funcall output-fn (subseq buffer 0 fill-pointer))
@@ -109,23 +114,13 @@
     (setf (flush-scheduled stream) nil))
   nil)
 
-#+(and sbcl sb-thread)
-(defmethod stream-force-output :around ((stream slime-output-stream))
-  ;; Workaround for deadlocks between the world-lock and auto-flush-thread
-  ;; buffer write lock.
-  ;;
-  ;; Another alternative would be to grab the world-lock here, but that's less
-  ;; future-proof, and could introduce other lock-ordering issues in the
-  ;; future.
-  (handler-case
-      (sb-sys:with-deadline (:seconds 0.1)
-        (call-next-method))
-    (sb-sys:deadline-timeout ()
-      nil)))
-
 (defmethod stream-force-output ((stream slime-output-stream))
   (stream-finish-output stream))
 
+(defmethod stream-finish-output ((stream slime-output-stream))
+  (unless (maybe-schedule-flush stream)
+    (%stream-finish-output stream)))
+
 (defmethod stream-fresh-line ((stream slime-output-stream))
   (with-slime-output-stream stream
     (cond ((zerop column) nil)
@@ -207,11 +202,17 @@
 (defimplementation make-auto-flush-thread (stream)
   (if (typep stream 'slime-output-stream)
       (setf (flush-thread stream)
-            (spawn (lambda () (auto-flush-loop stream 0.08 t))
+            (spawn (lambda () (auto-flush-loop stream 0.005 t #'%stream-finish-output))
                    :name "auto-flush-thread"))
       (spawn (lambda () (auto-flush-loop stream *auto-flush-interval*))
              :name "auto-flush-thread")))
 
+(defimplementation really-finish-output (stream)
+  (let ((stream (swank::real-output-stream stream)))
+    (if (typep stream 'slime-output-stream)
+        (%stream-finish-output stream)
+        (finish-output stream))))
+
 (defimplementation make-output-stream (write-string)
   (make-instance 'slime-output-stream :output-fn write-string))
 
