@@ -157,8 +157,8 @@
                       (not (load output-file)))))))))
 
 (defimplementation swank-compile-string (string &key buffer position filename
-                                                policy)
-  (declare (ignore filename policy))
+                                                line column policy)
+  (declare (ignore filename line column policy))
   (with-compilation-hooks ()
     (let ((*buffer-name* buffer)
           (*buffer-start-position* position)
